<?php
/**
 # jmootips Joomla! Plugin - Editor tooltip button
 # @author Joachim Schmidt - joachim.schmidt@jschmidt-systemberatung.de
 # @copyright Copyright (C) 2022 Joachim Schmidt. All rights reserved.
 # @license  http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 #
 # change activity:
 # 15.05.2013: Release V1.0.0 for Joomla 2.5 and 3.0
 # 01.03.2022: Corrected function for inserting tips based on selected text
 # 15.03.2022: Corrected javascripts interface for inserting tips based on selected text
 # 17.03.2022: Use web asset manager to build javascripts in Joomla V4
 # 20.03.3022: Correct $button->name to get tooltip icon when tinymce editor and joomla V3 used
 # 01.07.2022: change/correct deprecated function getDocument() for joomla v4.x
 # 10.10.2023: change/correct code to run with joomla 5 and php 8.2
 */
// no direct access
defined('_JEXEC') or die();

use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Object\CMSObject;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Session\Session;

# [\AllowDynamicProperties]
class plgButtonTipbutton extends CMSPlugin
{

     protected $autoloadLanguage = true;

    public function __construct (&$subject, $config)
    {
        parent::__construct($subject, $config);
        $this->loadLanguage();
    }

    function onDisplay ($name)
    {
        $app = Factory::getApplication();
        $doc = $app->getDocument();
        $editor = $app->get('editor');

        $wa = Factory::getApplication()->getDocument()->getWebAssetManager();
        if ($editor != 'jce' && $editor != 'arkeditor')
            $editor = 'none';
        $js = $this->buildScript($name, true, $editor);
        $wa->addInlineScript($js);

        $values = (array) Factory::getApplication()->getUserState('com_content.edit.article.id');
        $isEditingRecords = count($values);

        $hasAccess = $isEditingRecords;
        if (! $hasAccess)
            return;

        $link = "index.php?option=com_content&amp;view=articles&amp;layout=modal&amp;tmpl=component&amp;" . Session::getFormToken() . '=1' . "&amp;function=jSelectTooltip";

        $button = new CMSObject();
        $button->modal = true;
        $button->link = $link;
        $button->text = 'Tooltip';
        $button->title = Text::_('PLG_TIPBUTTON_BUTTON_TOOLTIP');
        $button->name = 'comments';

        $button->name = $this->_type . '_' . $this->_name;
        $button->icon = 'comments-2';
        $button->iconSVG = '<svg viewBox="-10 0 596 512">' .
                '<path d="M400 0q20 0 34 14t14 34v160q0 20 -14 34t-34 14h-182l-154 128v-128h-16q-20 0 -34 -14t-14 -34v-160q0 -20 14 -34t34 -14h352zM528 96q20 0 34 14t14 34v192q0 20 -14 34t-34 14h-80v128l-154 -128h-102v-64h126l98 87v-87h96v-160h-32v-64h48z" />' . '</svg>';

        $button->options = [
                'height' => '450px',
                'width' => '600px',
                'bodyHeight' => '70',
                'modalWidth' => '80'
        ];

        // $button->options = "{ handler: 'iframe', size: {x: 600, y: 450} }";

        return $button;
    }

    function buildScript ($name, $joomlav4, $editor = "")
    {
        $js = "
            ;(function(Joomla, window, document) {
               /* ---------------------------------------------------------------------- */
               /* jmootips Joomla! Plugin - Editor tooltip button                        */
               /* @author Joachim Schmidt - joachim.schmidt@jschmidt-systemberatung.de   */
               /* @copyright Copyright (C) 2022 Joachim Schmidt. All rights reserved.    */
               /* ---------------------------------------------------------------------- */
                
                function getSelection(myField) {
                  if (document.selection) {
                    myField.focus();
                   return document.selection.createRange();
                  } else if (myField.selectionStart || myField.selectionStart === 0) {
                   return myField.value.substring(myField.selectionStart, myField.selectionEnd);
                  } else {
                   return '';
                  }
                 }
               ";

        if ($editor == 'none')
            $js .= "
             function insertAtCursor(myField, myValue) {
              if (document.selection) {
               myField.focus();
               var sel = document.selection.createRange();
               sel.text = myValue;
              } else if (myField.selectionStart || myField.selectionStart === 0) {
                   myField.value = myField.value.substring(0, myField.selectionStart)
                + myValue
                + myField.value.substring(myField.selectionEnd, myField.value.length);
              } else {
               myField.value += myValue;
              }
             }

                window.jInsertEditorText = function(text, editor) {
               Joomla.editors.instances[editor].replaceSelection(text); }
                ";

        $js .= "
             window.getEditorText = function(text, editor) {
                           return  ( Joomla.editors.instances[editor].getSelection() );  }
                
                 document.addEventListener('DOMContentLoaded', function() {
                  var editors = document.querySelectorAll('.js-editor-none');
                
                  for(var i = 0, l = editors.length; i < l; i++) {
                   Joomla.editors.instances[editors[i].childNodes[0].id] = {
            'id': editors[i].childNodes[0].id,
            'element':  editors[i].childNodes[0],
            'getSelection': function () { return getSelection(this.element); },";
        if ($editor == 'none')
            $js .= "
                        'replaceSelection': function (text) { return insertAtCursor (this.element, text); },";

        $js .= "
            'onSave': function() { return ''; }
                    };
                   }
                 });

            }(Joomla, window, document));
                
            function jSelectTooltip( id, title, catid, object, link, lang ) {
                
               var editarea = document.getElementById('" . $name . "');
               var selection = (editarea.value).substring(editarea.selectionStart,editarea.selectionEnd);
               if (selection == '')
                {
                 try {
                       selection = getEditorText('tag', '" . $name . "');
                       if (selection.length > 100)
                       {
                          selection = '';
                          console.log('tipbutton error: selection string lenght > 100');
                       }
                     }
                 catch { selection = ''; }
               }
               if (selection == '' || selection == undefined)
               {
                  if ( confirm ('" . Text::_('PLG_TIPBUTTON_NO_SELECTION') . "') )
                    selection = title;
                  else
                  {";
        if (! $joomlav4)
            $js .= "
                    SqueezeBox.close();";
        else
            $js .= "
                    jQuery('#TooltipModal').modal('toggle');";
        $js .= "
                    return;
                  }
               }
               var tag = '{tip id=\"' +id +'\"}' + selection + '{/tip}';
            jInsertEditorText(tag, '" . $name . "');";

        if (! $joomlav4)
            $js .= "
               SqueezeBox.close();
            }\n";
        else
            $js .= "
               jQuery('#TooltipModal').modal('toggle');
            }\n";

        return $js;
    }
}